package com.ruoyi.bst.bonus.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.bst.areaJoin.domain.AreaJoinVO;
import com.ruoyi.bst.areaJoin.domain.enums.AreaJoinType;
import com.ruoyi.bst.areaJoin.service.AreaJoinService;
import com.ruoyi.bst.bonus.domain.Bonus;
import com.ruoyi.bst.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.bst.bonus.domain.enums.BonusBstType;
import com.ruoyi.bst.bonus.domain.enums.BonusStatus;
import com.ruoyi.bst.bonus.service.BonusConverter;
import com.ruoyi.bst.bonus.utils.BonusUtil;
import com.ruoyi.bst.channel.domain.ChannelVO;
import com.ruoyi.bst.channel.domain.enums.ChannelType;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.bo.OrderPayBO;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.iot.util.BatUtil;

@Service
public class BonusConverterImpl implements BonusConverter {

    @Autowired
    private AreaJoinService areaJoinService;

    @Override
    public List<Bonus> toBonusList(OrderPayBO bo) {
        if (bo == null) {
            return Collections.emptyList();
        }

        ChannelVO channel = bo.getChannel();
        if (channel == null) {
            return Collections.emptyList();
        }

        // 加盟信息，当车辆归属用户不是当前运营区的运营商时，才有加盟信息
        DeviceVO device = bo.getDevice();
        if (device != null && device.getMchId() != null && !Objects.equals(device.getMchId(), device.getAreaUserId())) {
            AreaJoinVO areaJoin = areaJoinService.selectByUserArea(device.getMchId(), device.getAreaId());
            bo.setAreaJoin(areaJoin);
        }

        // 若渠道为官方收款，则处理官方收款分成
        if (ChannelType.PLATFORM.getType().equals(channel.getType())) {
            return this.toBonusListPlatform(bo);
        } else {
            // TODO 待处理
            throw new ServiceException("暂不支持独立收款渠道");
        }
    }

    // 处理官方收款分成
    private List<Bonus> toBonusListPlatform(OrderPayBO bo) {

        // 获取基础分成列表
        List<Bonus> result = toBaseBonusListPlatform(bo.getDevice(), bo.getAreaJoin());

        // 设置分成金额
        Order order = bo.getOrder();
        BatUtil.partAmount(result, order.getPayAmount());

        // 设置其他基础数据
        String reason = "订单收入:" + order.getNo();
        for (Bonus bonus : result) {
            bonus.setReason(reason);
            bonus.setBstType(BonusBstType.ORDER.getType());
            bonus.setBstId(order.getId());
            bonus.setStatus(BonusStatus.INVALID.getStatus());
            bonus.setToBalance(true);
            bonus.setInvalidAmount(bonus.getAmount());
            bonus.setAreaId(order.getAreaId());
        }

        return result;
    }

    // 获取基础分成列表
    private List<Bonus> toBaseBonusListPlatform(DeviceVO device, AreaJoinVO areaJoin) {
        if (device == null) {
            return Collections.emptyList();
        }

        Long mchId = device.getMchId(); // 车辆归属用户
        Long areaUserId = device.getAreaUserId();   // 运营区归属用户
        Long areaAgentId = device.getAreaAgentId();
        BigDecimal totalPoint = BigDecimal.ZERO;    // 已分成百分比
        BigDecimal decimal100 = new BigDecimal(100);
        List<Bonus> result = new ArrayList<>();

        // 处理加盟分成
        if (areaJoin != null && areaJoin.getPoint() != null) {
            Long joinUserId = areaJoin.getUserId();  // 加盟用户ID
            boolean isAreaDevice = mchId != null && !Objects.equals(areaUserId, mchId); // 是否运营商的车辆
            String joinType = areaJoin.getType();   // 加盟方式
            // 有两种方式可获得分成：
            // 1.是运营车辆，且加盟方式为合伙人，则分成
            if (isAreaDevice && AreaJoinType.PARTNER.getCode().equals(joinType)) {
                // 合伙人分成
                Bonus bonus = BonusUtil.getInitBonus(BonusArrivalType.PARTNER, joinUserId, areaJoin.getUserName(), areaJoin.getPoint());
                if (bonus != null) {
                    result.add(bonus);
                    totalPoint = totalPoint.add(bonus.getPoint());
                }
            } 
            // 2.不是运营车辆，且加盟方式为加盟商，则分成
            else if (!isAreaDevice && AreaJoinType.JOIN.getCode().equals(joinType)) {
                // 加盟商分成
                Bonus bonus = BonusUtil.getInitBonus(BonusArrivalType.JOIN, joinUserId, areaJoin.getUserName(), areaJoin.getPoint());
                if (bonus != null) {
                    result.add(bonus);
                    totalPoint = totalPoint.add(bonus.getPoint());
                }
            }
        }

        
        // 运营商分成
        if (areaUserId != null && device.getAreaUserPoint() != null ) {
            Bonus bonus = BonusUtil.getInitBonus(BonusArrivalType.MCH, areaUserId, device.getAreaUserName(), device.getAreaUserPoint().subtract(totalPoint));
            if (bonus != null) {
                result.add(bonus);
                totalPoint = totalPoint.add(bonus.getPoint());
            }
        }

        // 代理商分成
        if (areaAgentId != null && device.getAreaAgentPoint() != null) {
            Bonus bonus = BonusUtil.getInitBonus(BonusArrivalType.AGENT, areaAgentId, device.getAreaAgentName(), device.getAreaAgentPoint().subtract(totalPoint));
            if (bonus != null) {
                result.add(bonus);
                totalPoint = totalPoint.add(bonus.getPoint());
            }
        }
        
        // 平台分成
        if (totalPoint.compareTo(decimal100) < 0) {
            Bonus bonus = BonusUtil.getInitBonus(BonusArrivalType.PLATFORM, 1L, "创特科技", decimal100.subtract(totalPoint));
            if (bonus != null) {
                result.add(bonus);
                totalPoint = totalPoint.add(bonus.getPoint());
            }
        }
        
        // 判断分成是否正确
        if (totalPoint.compareTo(decimal100) != 0) {
            throw new ServiceException("分成计算错误");
        }

        return result;
    }

}
